@@ -8,6 +8,7 @@
 # @author Davide Brunato <brunato@sissa.it>
 #
 import json
+import copy
 from io import IOBase, TextIOBase
 from typing import Any, Dict, List, Optional, Type, Union, Tuple, \
     IO, BinaryIO, TextIO, Iterator
@@ -73,10 +74,10 @@ def get_context(xml_document: Union[XMLSourceType, XMLResource],
                 return resource, cls(schema, **kwargs)
 
     if schema is None:
-        if cls.meta_schema is not None and \
-                (XSI_TYPE in resource.root.attrib or XSD_NAMESPACE in resource.namespace):
+        if XSD_NAMESPACE == resource.namespace:
+            assert cls.meta_schema is not None
             return resource, cls.meta_schema
-        elif dummy_schema:
+        elif dummy_schema or XSI_TYPE in resource.root.attrib:
             return resource, get_dummy_schema(resource.root.tag, cls)
         else:
             msg = "cannot get a schema for XML data, provide a schema argument"
@@ -542,6 +543,9 @@ class XmlDocument(XMLResource):
     :param defuse: the defuse mode for base :class:`xmlschema.XMLResource` initialization.
     :param timeout: the timeout for base :class:`xmlschema.XMLResource` initialization.
     :param lazy: the lazy mode for base :class:`xmlschema.XMLResource` initialization.
+    :param use_location_hints: for default, in case a schema instance has \
+    to be built, uses also schema locations hints provided within XML data. \
+    Set this option to `False` to ignore these schema location hints.
     """
     schema: Optional[XMLSchemaBase] = None
     _fallback_schema: Optional[XMLSchemaBase] = None
@@ -598,12 +602,12 @@ class XmlDocument(XMLResource):
 
             if self.schema is None:
                 if XSI_TYPE in self._root.attrib:
-                    self.schema = cls.meta_schema
+                    self.schema = get_dummy_schema(self._root.tag, cls)
                 elif validation != 'skip':
                     msg = "cannot get a schema for XML data, provide a schema argument"
                     raise XMLSchemaValueError(msg)
                 else:
-                    self._fallback_schema = get_dummy_schema(self.root.tag, cls)
+                    self._fallback_schema = get_dummy_schema(self._root.tag, cls)
 
         if self.schema is None:
             pass
@@ -616,7 +620,7 @@ class XmlDocument(XMLResource):
 
     def parse(self, source: XMLSourceType, lazy: LazyType = False) -> None:
         super(XmlDocument, self).parse(source, lazy)
-        self.namespaces = self.get_namespaces(self._namespaces)
+        self.namespaces = self.get_namespaces()
 
         if self.schema is None:
             pass
@@ -625,40 +629,38 @@ class XmlDocument(XMLResource):
         elif self.validation == 'lax':
             self.errors = [e for e in self.schema.iter_errors(self, namespaces=self.namespaces)]
 
+    def get_namespaces(self, namespaces: Optional[NamespacesType] = None,
+                       root_only: Optional[bool] = None) -> NamespacesType:
+        if not self._namespaces:
+            _namespaces = namespaces
+        elif not namespaces:
+            _namespaces = self._namespaces
+        else:
+            _namespaces = copy.copy(self._namespaces)
+            _namespaces.update(namespaces)
+
+        return super().get_namespaces(_namespaces, root_only)
+
     def getroot(self) -> ElementType:
         """Get the root element of the XML document."""
         return self._root
 
     def get_etree_document(self) -> Any:
         """
-        The resource as ElementTree XML document. If the resource is lazy raises a resource error.
+        The resource as ElementTree XML document. If the resource is lazy
+        raises a resource error.
         """
         if is_etree_document(self._source):
             return self._source
         elif self._lazy:
-            raise XMLResourceError("cannot create an ElementTree from a lazy resource")
+            raise XMLResourceError(
+                "cannot create an ElementTree instance from a lazy XML resource"
+            )
         elif hasattr(self._root, 'nsmap'):
             return self._root.getroottree()  # type: ignore[attr-defined]
         else:
             return ElementTree.ElementTree(self._root)
 
-    def tostring(self, indent: str = '', max_lines: Optional[int] = None,
-                 spaces_for_tab: int = 4, xml_declaration: bool = False,
-                 encoding: str = 'unicode', method: str = 'xml') -> str:
-        if self._lazy:
-            raise XMLResourceError("cannot serialize a lazy XML document")
-
-        _string = etree_tostring(
-            elem=self._root,
-            namespaces=self.namespaces,
-            xml_declaration=xml_declaration,
-            encoding=encoding,
-            method=method
-        )
-        if isinstance(_string, bytes):
-            return _string.decode('utf-8')
-        return _string
-
     def decode(self, **kwargs: Any) -> DecodeType[Any]:
         """
         Decode the XML document to a nested Python dictionary.
@@ -731,7 +733,7 @@ class XmlDocument(XMLResource):
               default_namespace: Optional[str] = None, method: str = "xml") -> None:
         """Serialize an XML resource to a file. Cannot be used with lazy resources."""
         if self._lazy:
-            raise XMLResourceError("cannot serialize a lazy XML document")
+            raise XMLResourceError("cannot serialize a lazy XML resource")
 
         kwargs: Dict[str, Any] = {
             'xml_declaration': xml_declaration,
